@@ -20,14 +20,13 @@
 __author__ = 'Sandro Santilli'
 __date__ = 'May 2017'
 __copyright__ = '(C) 2017, Sandro Santilli'
-# This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import os
 import qgis
 from qgis.testing import start_app, unittest
 from qgis.core import QgsDataSourceUri
 from qgis.utils import iface
+from qgis.PyQt.QtCore import QObject
 
 start_app()
 
@@ -50,10 +49,15 @@ class TestDBManagerPostgisConnector(unittest.TestCase):
         connector._close_cursor(r)
         return val
 
-    # See https://issues.qgis.org/issues/16625
-    # and https://issues.qgis.org/issues/10600
+    # See https://github.com/qgis/QGIS/issues/24525
+    # and https://github.com/qgis/QGIS/issues/19005
     def test_dbnameLessURI(self):
-        c = PostGisDBConnector(QgsDataSourceUri())
+
+        obj = QObject() # needs to be kept alive
+        obj.connectionName = lambda: 'fake'
+        obj.providerName = lambda: 'postgres'
+
+        c = PostGisDBConnector(QgsDataSourceUri(), obj)
         self.assertIsInstance(c, PostGisDBConnector)
         uri = c.uri()
 
